@@ -1,11 +1,11 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
 #   [MS-SRVS] Interface implementation
 #
@@ -14,9 +14,12 @@
 #   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
-#   They are located at the end of this file. 
+#   They are located at the end of this file.
 #   Helper functions start with "h"<name of the call>.
-#   There are test cases for them too. 
+#   There are test cases for them too.
+#
+# Author:
+#   Alberto Solino (@agsolino)
 #
 from __future__ import division
 from __future__ import print_function
@@ -1781,7 +1784,7 @@ class WCHAR_ARRAY(NDRSTRUCT):
         else:
             return NDR.__getitem__(self,key)
 
-    def getDataLen(self, data):
+    def getDataLen(self, data, offset=0):
         return self["ActualCount"]*2 
 
 
@@ -1889,7 +1892,7 @@ class LPSERVER_TRANSPORT_INFO_2_ARRAY(NDRPOINTER):
 
 # 2.2.4.96 SERVER_TRANSPORT_INFO_3
 class PASSWORD_ARRAY(NDRUniFixedArray):
-    def getDataLen(self, data):
+    def getDataLen(self, data, offset=0):
         return 256
 
 class SERVER_TRANSPORT_INFO_3(NDRSTRUCT):
