@@ -1,11 +1,11 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
 #   [MS-DRSR] Directory Replication Service (DRS) DRSUAPI Interface implementation
 #
@@ -14,10 +14,14 @@
 #   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
-#   They are located at the end of this file. 
+#   They are located at the end of this file.
 #   Helper functions start with "h"<name of the call>.
-#   There are test cases for them too. 
+#   There are test cases for them too.
 #
+# Author:
+#   Alberto Solino (@agsolino)
+#
+
 from __future__ import division
 from __future__ import print_function
 from builtins import bytes
@@ -625,7 +629,7 @@ class DSNAME(NDRSTRUCT):
         ('NameLen',ULONG),
         ('StringName', WCHAR_ARRAY),
     )
-    def getDataLen(self, data):
+    def getDataLen(self, data, offset=0):
         return self['NameLen']
     def getData(self, soFar = 0):
         return NDRSTRUCT.getData(self, soFar)
