@@ -1,6 +1,8 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
@@ -11,17 +13,17 @@
 #   so you understand what the call does, and then read the test case located
 #   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
-# ToDo: 
-# [ ] Take out all the security provider stuff out of here (e.g. RPC_C_AUTHN_WINNT)
-#     and put it elsewhere. This will make the coder cleaner and easier to add 
-#     more SSP (e.g. NETLOGON)
+# ToDo:
+#   [ ] Take out all the security provider stuff out of here (e.g. RPC_C_AUTHN_WINNT)
+#       and put it elsewhere. This will make the coder cleaner and easier to add
+#       more SSP (e.g. NETLOGON)
 # 
 
 import logging
 import socket
 import sys
 from binascii import unhexlify
-from Crypto.Cipher import ARC4
+from Cryptodome.Cipher import ARC4
 
 from impacket import ntlm, LOG
 from impacket.structure import Structure,pack,unpack
@@ -53,6 +55,7 @@ MSRPC_AUTH3     = 0x10
 MSRPC_SHUTDOWN  = 0x11
 MSRPC_CO_CANCEL = 0x12
 MSRPC_ORPHANED  = 0x13
+MSRPC_RTS       = 0x14
 
 # MS/RPC Packet Flags
 PFC_FIRST_FRAG     = 0x01
