@@ -1,11 +1,11 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2020 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
 #   [MS-SMB2] Protocol Implementation (SMB2 and SMB3)
 #   As you might see in the code, it's implemented strictly following
@@ -14,16 +14,20 @@
 #   same to self._Session in the context of this library ) but
 #   it certainly helps following the document way easier.
 #
-# ToDo:
-# [X] Implement SMB2_CHANGE_NOTIFY
-# [X] Implement SMB2_QUERY_INFO
-# [X] Implement SMB2_SET_INFO
-# [ ] Implement SMB2_OPLOCK_BREAK
-# [X] Implement SMB3 signing
-# [X] Implement SMB3 encryption
-# [ ] Add more backward compatible commands from the smb.py code
-# [ ] Fix up all the 'ToDo' comments inside the code
+# Author:
+#   Alberto Solino (@agsolino)
 #
+# ToDo:
+#   [X] Implement SMB2_CHANGE_NOTIFY
+#   [X] Implement SMB2_QUERY_INFO
+#   [X] Implement SMB2_SET_INFO
+#   [ ] Implement SMB2_OPLOCK_BREAK
+#   [X] Implement SMB3 signing
+#   [X] Implement SMB3 encryption
+#   [ ] Add more backward compatible commands from the smb.py code
+#   [ ] Fix up all the 'ToDo' comments inside the code
+#
+
 from __future__ import division
 from __future__ import print_function
 
@@ -36,7 +40,7 @@ from six import indexbytes, b
 from binascii import a2b_hex
 from contextlib import contextmanager
 from pyasn1.type.univ import noValue
-from Crypto.Cipher import AES
+from Cryptodome.Cipher import AES
 
 from impacket import nmb, ntlm, uuid, crypto
 from impacket.smb3structs import *
@@ -178,6 +182,9 @@ class SMB3:
             # Outside the protocol
             'ServerIP'                 : '',    #
             'ClientName'               : '',    #
+            #GSSoptions (MutualAuth and Delegate)
+            'GSSoptions'               : {},
+            'PreauthIntegrityHashValue': a2b_hex(b'0'*128)
         }
 
         self._Session = {
@@ -274,8 +281,15 @@ class SMB3:
         for i in list(self._Session.items()):
             print("%-40s : %s" % i)
 
+    def __UpdateConnectionPreAuthHash(self, data):
+        from Cryptodome.Hash import SHA512
+        calculatedHash =  SHA512.new()
+        calculatedHash.update(self._Connection['PreauthIntegrityHashValue'])
+        calculatedHash.update(data)
+        self._Connection['PreauthIntegrityHashValue'] = calculatedHash.digest()
+
     def __UpdatePreAuthHash(self, data):
-        from Crypto.Hash import SHA512
+        from Cryptodome.Hash import SHA512
         calculatedHash =  SHA512.new()
         calculatedHash.update(self._Session['PreauthIntegrityHashValue'])
         calculatedHash.update(data)
@@ -385,6 +399,14 @@ class SMB3:
                 packet['Flags'] = SMB2_FLAGS_SIGNED
                 self.signSMB(packet)
 
+        if packet['Command'] is SMB2_NEGOTIATE:
+            data = packet.getData()
+            self.__UpdateConnectionPreAuthHash(data)
+            self._Session['CalculatePreAuthHash'] = False
+
+        if packet['Command'] is SMB2_SESSION_SETUP:
+            self._Session['CalculatePreAuthHash'] = True
+
         if (self._Session['SessionFlags'] & SMB2_SESSION_FLAG_ENCRYPT_DATA) or ( packet['TreeID'] != 0 and self._Session['TreeConnectTable'][packet['TreeID']]['EncryptData'] is True):
             plainText = packet.getData()
             transformHeader = SMB2_TRANSFORM_HEADER()
@@ -540,7 +562,7 @@ class SMB3:
             if ans.isValidAnswer(STATUS_SUCCESS):
                 negResp = SMB2Negotiate_Response(ans['Data'])
                 if negResp['DialectRevision']  == SMB2_DIALECT_311:
-                    self.__UpdatePreAuthHash(ans.rawData)
+                    self.__UpdateConnectionPreAuthHash(ans.rawData)
 
         self._Connection['MaxTransactSize']   = min(0x100000,negResp['MaxTransactSize'])
         self._Connection['MaxReadSize']       = min(0x100000,negResp['MaxReadSize'])
@@ -586,7 +608,7 @@ class SMB3:
             self.__TGT,
             self.__TGS)
 
-    def kerberosLogin(self, user, password, domain = '', lmhash = '', nthash = '', aesKey='', kdcHost = '', TGT=None, TGS=None):
+    def kerberosLogin(self, user, password, domain = '', lmhash = '', nthash = '', aesKey='', kdcHost = '', TGT=None, TGS=None, mutualAuth=False):
         # If TGT or TGS are specified, they are in the form of:
         # TGS['KDC_REP'] = the response from the server
         # TGS['cipher'] = the cipher used
@@ -680,7 +702,13 @@ class SMB3:
         apReq['pvno'] = 5
         apReq['msg-type'] = int(constants.ApplicationTagNumbers.AP_REQ.value)
 
+        #Handle mutual authentication
         opts = list()
+
+        if mutualAuth == True:
+            from impacket.krb5.constants import APOptions
+            opts.append(constants.APOptions.mutual_required.value)
+
         apReq['ap-options'] = constants.encodeFlags(opts)
         seq_set(apReq,'ticket', ticket.to_asn1)
 
@@ -715,6 +743,9 @@ class SMB3:
         packet['Command'] = SMB2_SESSION_SETUP
         packet['Data']    = sessionSetup
 
+        #Initiate session preauth hash
+        self._Session['PreauthIntegrityHashValue'] = self._Connection['PreauthIntegrityHashValue']
+
         packetID = self.sendSMB(packet)
         ans = self.recvSMB(packetID)
         if ans.isValidAnswer(STATUS_SUCCESS):
@@ -723,7 +754,47 @@ class SMB3:
             self._Session['UserCredentials'] = (user, password, domain, lmhash, nthash)
             self._Session['Connection']      = self._NetBIOSSession.get_socket()
 
-            self._Session['SessionKey']  = sessionKey.contents[:16]
+
+            if mutualAuth == True:
+                #Lets get the session key in the AP_REP
+                from impacket.krb5.asn1 import AP_REP, EncAPRepPart
+                from impacket.krb5.crypto import Key, _enctype_table
+                smbSessSetupResp = SMB2SessionSetup_Response(ans['Data'])
+
+                #in [KILE] 3.1.1.2:
+                #    The subkey in the EncAPRepPart of the KRB_AP_REP message is used as the session key when
+                #    MutualAuthentication is requested. (The KRB_AP_REP message and its fields are defined in [RFC4120]
+                #    section 5.5.2.) When DES and RC4 are used, the implementation is as described in [RFC1964]. With
+                #    DES and RC4, the subkey in the KRB_AP_REQ message can be used as the session key, as it is the
+                #    same as the subkey in KRB_AP_REP message; however when AES is used (see [RFC4121]), the
+                #    subkeys are different and the subkey in the KRB_AP_REP is used. (The KRB_AP_REQ message is
+                #    defined in [RFC4120] section 5.5.1).
+                negTokenResp = SPNEGO_NegTokenResp(smbSessSetupResp['Buffer'])
+
+                #TODO: Parse ResponseToken as krb5Blob depending on the supported mech indicated in the negTokenResp
+                ap_rep = decoder.decode(negTokenResp['ResponseToken'][16:], asn1Spec=AP_REP())[0]
+
+                if cipher.enctype != ap_rep['enc-part']['etype']:
+                    raise Exception('Unable to decrypt AP_REP: cipher does not match TGS session key')
+
+                # Key Usage 12
+                # AP-REP encrypted part (includes application session
+                # subkey), encrypted with the application session key
+                # (Section 5.5.2)
+                cipherText = ap_rep['enc-part']['cipher']
+                plainText = cipher.decrypt(sessionKey, 12, cipherText)
+
+                encAPRepPart = decoder.decode(plainText, asn1Spec = EncAPRepPart())[0]
+
+                apCipher = _enctype_table[int(encAPRepPart['subkey']['keytype'])]()
+                apSessionKey = Key(apCipher.enctype, encAPRepPart['subkey']['keyvalue'].asOctets())
+
+                sequenceNumber = int(encAPRepPart['seq-number'])
+                self._Session['SessionKey'] = apSessionKey.contents
+
+            else:
+                self._Session['SessionKey']  = sessionKey.contents[:16]
+
             if self._Session['SigningRequired'] is True and self._Connection['Dialect'] >= SMB2_DIALECT_30:
                 # If Connection.Dialect is "3.1.1", the case-sensitive ASCII string "SMBSigningKey" as the label;
                 # otherwise, the case - sensitive ASCII string "SMB2AESCMAC" as the label.
