@@ -1,17 +1,21 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2018 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
 # Description:
-#  Network packet codecs basic building blocks.
-#  Low-level packet codecs for various Internet protocols.
+#   Network packet codecs basic building blocks.
+#   Low-level packet codecs for various Internet protocols.
 #
 # Author:
-#  Javier Burroni (javier)
-#  Bruce Leidl (brl)
-#  Javier Kohen (jkohen)
+#   Javier Burroni (javier)
+#   Bruce Leidl (brl)
+#   Javier Kohen (jkohen)
+#
+
 from __future__ import division
 from __future__ import print_function
 import array
@@ -22,6 +26,16 @@ import sys
 from binascii import hexlify
 from functools import reduce
 
+# Alias function for compatibility with both Python <3.2 `tostring` and `fromstring` methods, and
+# Python >=3.2 `tobytes` and `tostring`
+if sys.version_info[0] >= 3 and sys.version_info[1] >= 2:
+    array_tobytes = lambda array_object: array_object.tobytes()
+    array_frombytes = lambda array_object, bytes: array_object.frombytes(bytes)
+else:
+    array_tobytes = lambda array_object: array_object.tostring()
+    array_frombytes = lambda array_object, bytes: array_object.fromstring(bytes)
+
+
 """Classes to build network packets programmatically.
 
 Each protocol layer is represented by an object, and these objects are
@@ -60,7 +74,7 @@ class PacketBuffer(object):
 
     def get_buffer_as_string(self):
         "Returns the packet buffer as a string object"
-        return self.__bytes.tostring()
+        return array_tobytes(self.__bytes)
 
     def get_bytes(self):
         "Returns the packet buffer as an array"
@@ -97,7 +111,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+2]
-        (value,) = struct.unpack(order + 'H', bytes.tostring())
+        (value,) = struct.unpack(order + 'H', array_tobytes(bytes))
         return value
 
     def set_long(self, index, value, order = '!'):
@@ -116,7 +130,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+4]
-        (value,) = struct.unpack(order + 'L', bytes.tostring())
+        (value,) = struct.unpack(order + 'L', array_tobytes(bytes))
         return value
 
     def set_long_long(self, index, value, order = '!'):
@@ -135,7 +149,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+8]
-        (value,) = struct.unpack(order + 'Q', bytes.tostring())
+        (value,) = struct.unpack(order + 'Q', array_tobytes(bytes))
         return value
 
 
@@ -146,7 +160,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+4]
-        return socket.inet_ntoa(bytes.tostring())
+        return socket.inet_ntoa(array_tobytes(bytes))
 
     def set_ip_address(self, index, ip_string):
         "Set 4-byte value at 'index' from 'ip_string'"
@@ -196,7 +210,7 @@ class PacketBuffer(object):
 
         diff = index + size - curlen
         if diff > 0:
-            self.__bytes.fromstring('\0' * diff)
+            array_frombytes(self.__bytes, b'\0' * diff)
             if orig_index < 0:
                 orig_index -= diff
 
@@ -719,7 +733,7 @@ class LinuxSLL(Header):
 
     def get_addr(self):
         "Returns the sender's address field"
-        return self.get_bytes()[6:14].tostring()
+        return array_tobytes(self.get_bytes()[6:14])
 
     def set_ether_type(self, aValue):
         "Set ethernet data type field to 'aValue'"
@@ -797,7 +811,7 @@ class IP(Header):
         # Pad to a multiple of 4 bytes
         num_pad = (4 - (len(my_bytes) % 4)) % 4
         if num_pad:
-            my_bytes.fromstring(b"\0"* num_pad)
+            array_frombytes(my_bytes, b"\0" * num_pad)
 
         # only change ip_hl value if options are present
         if len(self.__option_list):
@@ -809,9 +823,9 @@ class IP(Header):
             self.set_ip_sum(self.compute_checksum(my_bytes))
 
         if child_data is None:
-            return my_bytes.tostring()
+            return array_tobytes(my_bytes)
         else:
-            return my_bytes.tostring() + child_data
+            return array_tobytes(my_bytes) + child_data
 
 
 
@@ -835,7 +849,7 @@ class IP(Header):
 
         size_str = struct.pack("!H", tmp_size)
 
-        pseudo_buf.fromstring(size_str)
+        array_frombytes(pseudo_buf, size_str)
         return pseudo_buf
 
     def add_option(self, option):
@@ -1296,7 +1310,7 @@ class UDP(Header):
             buffer += self.get_bytes()
             data = self.get_data_as_string()
             if(data):
-                buffer.fromstring(data)
+                array_frombytes(buffer, data)
             self.set_uh_sum(self.compute_checksum(buffer))
 
     def get_header_size(self):
@@ -1486,7 +1500,7 @@ class TCP(Header):
 
         data = self.get_data_as_string()
         if(data):
-            buffer.fromstring(data)
+            array_frombytes(buffer, data)
 
         res = self.compute_checksum(buffer)
 
@@ -1505,9 +1519,9 @@ class TCP(Header):
         data = self.get_data_as_string()
 
         if data:
-            return bytes.tostring() + data
+            return array_tobytes(bytes) + data
         else:
-            return bytes.tostring()
+            return array_tobytes(bytes)
 
     def load_header(self, aBuffer):
         self.set_bytes_from_string(aBuffer[:20])
@@ -1562,7 +1576,7 @@ class TCP(Header):
             op_buf += op.get_bytes()
         num_pad = (4 - (len(op_buf) % 4)) % 4
         if num_pad:
-            op_buf.fromstring("\0" * num_pad)
+            array_frombytes(op_buf, "\0" * num_pad)
         return op_buf
 
     def __str__(self):
