@@ -1,29 +1,30 @@
-#!/usr/local/bin/python3.8
-#!/usr/local/bin/python3.8
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.12
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author:
-#  Alberto Solino (@agsolino)
-#
 # Description:
-#    This script will attempt to list and get TGTs for those users that have the property
-#    'Do not require Kerberos preauthentication' set (UF_DONT_REQUIRE_PREAUTH).
-#    For those users with such configuration, a John The Ripper output will be generated so
-#    you can send it for cracking.
+#   This script will attempt to list and get TGTs for those users that have the property
+#   'Do not require Kerberos preauthentication' set (UF_DONT_REQUIRE_PREAUTH).
+#   For those users with such configuration, a John The Ripper output will be generated so
+#   you can send it for cracking.
 #
-#    Original credit for this technique goes to @harmj0y:
-#    https://www.harmj0y.net/blog/activedirectory/roasting-as-reps/
-#    Related work by Geoff Janjua:
-#    https://www.exumbraops.com/layerone2016/party
+#   Original credit for this technique goes to @harmj0y:
+#   https://www.harmj0y.net/blog/activedirectory/roasting-as-reps/
+#   Related work by Geoff Janjua:
+#   https://www.exumbraops.com/layerone2016/party
 #
-#    For usage instructions run the script with no parameters
+#   For usage instructions run the script with no parameters.
 #
-# ToDo:
+# Author:
+#   Alberto Solino (@agsolino)
 #
+
 from __future__ import division
 from __future__ import print_function
 import argparse
@@ -39,6 +40,7 @@ from pyasn1.type.univ import noValue
 from impacket import version
 from impacket.dcerpc.v5.samr import UF_ACCOUNTDISABLE, UF_DONT_REQUIRE_PREAUTH
 from impacket.examples import logger
+from impacket.examples.utils import parse_credentials
 from impacket.krb5 import constants
 from impacket.krb5.asn1 import AS_REQ, KERB_PA_PAC_REQUEST, KRB_ERROR, AS_REP, seq_set, seq_set_iter
 from impacket.krb5.kerberosv5 import sendReceive, KerberosError
@@ -200,6 +202,10 @@ class GetUserNoPreAuth:
             fd.write(entry + '\n')
 
     def run(self):
+        if self.__usersFile:
+            self.request_users_file_TGTs()
+            return
+
         if self.__doKerberos:
             target = self.getMachineName()
         else:
@@ -208,13 +214,8 @@ class GetUserNoPreAuth:
             else:
                 target = self.__domain
 
-        if self.__usersFile:
-            self.request_users_file_TGTs()
-            return
-
-
         # Are we asked not to supply a password?
-        if self.__no_pass is True:
+        if self.__doKerberos is False and self.__no_pass is True:
             # Yes, just ask the TGT and exit
             logging.info('Getting TGT for %s' % self.__username)
             entry = self.getTGT(self.__username)
@@ -347,8 +348,8 @@ if __name__ == '__main__':
                                   "'Do not require Kerberos preauthentication' set and export their TGTs for cracking")
 
     parser.add_argument('target', action='store', help='domain/username[:password]')
-    parser.add_argument('-request', action='store_true', default='False', help='Requests TGT for users and output them '
-                                                                               'in JtR/hashcat format (default False)')
+    parser.add_argument('-request', action='store_true', default=False, help='Requests TGT for users and output them '
+                                                                             'in JtR/hashcat format (default False)')
     parser.add_argument('-outputfile', action='store',
                         help='Output filename to write ciphers in JtR/hashcat format')
 
@@ -404,16 +405,7 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
-    # This is because I'm lazy with regex
-    # ToDo: We need to change the regex to fullfil domain/username[:password]
-    targetParam = options.target+'@'
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(targetParam).groups('')
-
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    domain, username, password = parse_credentials(options.target)
 
     if domain == '':
         logging.critical('Domain should be specified!')
@@ -426,6 +418,10 @@ if __name__ == '__main__':
     if options.aesKey is not None:
         options.k = True
 
+    if options.k is False and options.no_pass is True and username == '' and options.usersfile is None:
+        logging.critical('If the -no-pass option was specified, but Kerberos (-k) is not used, then a username or the -usersfile option should be specified!')
+        sys.exit(1)
+
     if options.outputfile is not None:
         options.request = True
 
