@@ -1,5 +1,14 @@
-#!/usr/local/bin/python3.8
-#!/usr/local/bin/python3.8
+#!/usr/local/bin/python3.13
+#!/usr/local/bin/python3.13
+# Impacket - Collection of Python classes for working with network protocols.
+#
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
+# of the Apache Software License. See the accompanying LICENSE file
+# for more information.
+#
+
 import uncrc32
 
 try:
@@ -9,7 +18,7 @@ except ImportError:
 
 from impacket import ImpactPacket
 from impacket import ImpactDecoder
-from impacket.ImpactPacket import TCPOption
+from impacket.ImpactPacket import TCPOption, array_tobytes
 from impacket.examples import logger
 from impacket.examples import os_ident
 
@@ -355,7 +364,7 @@ class UDPCommandResponder(OpenUDPResponder):
           #in_onion[O_UDP].get_uh_dport() == self.port)
 
    def buildAnswer(self, in_onion):
-       cmd = in_onion[O_UDP_DATA].get_bytes().tostring()
+       cmd = array_tobytes(in_onion[O_UDP_DATA].get_bytes())
        if cmd[:4] == 'cmd:': cmd = cmd[4:].strip()
        print("Got command: %r" % cmd)
 
@@ -1116,4 +1125,4 @@ if __name__ == '__main__':
 # [-] ??? (RUL) Length of return UDP packet is correct
 
 # doas nmap -O 127.0.0.2 -p 22,111,89
-# doas /usr/local/bin/python3.8 nmapAnswerMachine.py -i em0 -p 192.168.66.254 -f 'Sun Solaris 9 (SPARC)'
+# doas /usr/local/bin/python3.13 nmapAnswerMachine.py -i em0 -p 192.168.66.254 -f 'Sun Solaris 9 (SPARC)'
