@@ -1,21 +1,26 @@
-#!/usr/local/bin/python3.8
-#!/usr/local/bin/python3.8
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.9
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# PSEXEC like functionality example using RemComSvc (https://github.com/kavika13/RemCom)
+# Description:
+#   PSEXEC like functionality example using RemComSvc (https://github.com/kavika13/RemCom)
 #
 # Author:
-#  beto (@agsolino)
+#   beto (@agsolino)
 #
 # Reference for:
-#  DCE/RPC and SMB.
+#   DCE/RPC and SMB.
+#
 
 import sys
 import os
+import re
 import cmd
 import logging
 from threading import Thread, Lock
@@ -31,8 +36,10 @@ from impacket.smbconnection import SMBConnection
 from impacket.dcerpc.v5 import transport
 from impacket.structure import Structure
 from impacket.examples import remcomsvc, serviceinstall
+from impacket.examples.utils import parse_target
 from impacket.krb5.keytab import Keytab
 
+CODEC = sys.stdout.encoding
 
 class RemComMessage(Structure):
     structure = (
@@ -50,9 +57,9 @@ class RemComResponse(Structure):
         ('ReturnCode','<L=0'),
     )
 
-RemComSTDOUT         = "RemCom_stdout"
-RemComSTDIN          = "RemCom_stdin"
-RemComSTDERR         = "RemCom_stderr"
+RemComSTDOUT = "RemCom_stdout"
+RemComSTDIN = "RemCom_stdin"
+RemComSTDERR = "RemCom_stderr"
 
 lock = Lock()
 
@@ -79,18 +86,15 @@ class PSEXEC:
             self.__lmhash, self.__nthash = hashes.split(':')
 
     def run(self, remoteName, remoteHost):
-
         stringbinding = r'ncacn_np:%s[\pipe\svcctl]' % remoteName
         logging.debug('StringBinding %s'%stringbinding)
         rpctransport = transport.DCERPCTransportFactory(stringbinding)
         rpctransport.set_dport(self.__port)
         rpctransport.setRemoteHost(remoteHost)
-
         if hasattr(rpctransport, 'set_credentials'):
             # This method exists only for selected protocol sequences.
             rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
                                          self.__nthash, self.__aesKey)
-
         rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
         self.doStuff(rpctransport)
 
@@ -138,7 +142,7 @@ class PSEXEC:
                 installService = serviceinstall.ServiceInstall(rpctransport.get_smb_connection(), remcomsvc.RemComSvc(), self.__serviceName, self.__remoteBinaryName)
             else:
                 try:
-                    f = open(self.__exeFile)
+                    f = open(self.__exeFile, 'rb')
                 except Exception as e:
                     logging.critical(str(e))
                     sys.exit(1)
@@ -217,6 +221,7 @@ class PSEXEC:
             sys.stdout.flush()
             sys.exit(1)
 
+
 class Pipes(Thread):
     def __init__(self, transport, pipe, permissions, share=None):
         Thread.__init__(self)
@@ -262,26 +267,117 @@ class RemoteStdOutPipe(Pipes):
 
     def run(self):
         self.connectPipe()
-        while True:
-            try:
-                ans = self.server.readFile(self.tid,self.fid, 0, 1024)
-            except:
-                pass
-            else:
+
+        global LastDataSent
+
+        if PY3:
+            __stdoutOutputBuffer, __stdoutData = b"", b""
+
+            while True:
                 try:
-                    global LastDataSent
-                    if ans != LastDataSent:
-                        sys.stdout.write(ans.decode('cp437'))
-                        sys.stdout.flush()
-                    else:
-                        # Don't echo what I sent, and clear it up
-                        LastDataSent = ''
-                    # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
-                    # it will give false positives tho.. we should find a better way to handle this.
-                    if LastDataSent > 10:
-                        LastDataSent = ''
+                    stdout_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
                 except:
                     pass
+                else:
+                    try:
+                        if stdout_ans != LastDataSent:
+                            if len(stdout_ans) != 0:
+                                # Append new data to the buffer while there is data to read
+                                __stdoutOutputBuffer += stdout_ans
+
+                        promptRegex = b'([a-zA-Z]:[\\\/])((([a-zA-Z0-9 -\.]*)[\\\/]?)+(([a-zA-Z0-9 -\.]+))?)?>$'
+
+                        endsWithPrompt = bool(re.match(promptRegex, __stdoutOutputBuffer) is not None)
+                        if endsWithPrompt == True:
+                            # All data, we shouldn't have encoding errors
+                            # Adding a space after the prompt because it's beautiful
+                            __stdoutData = __stdoutOutputBuffer + b" "
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = b""
+
+                            # print("[+] endsWithPrompt")
+                            # print(" | __stdoutData:",__stdoutData)
+                            # print(" | __stdoutOutputBuffer:",__stdoutOutputBuffer)
+                        elif b'\n' in __stdoutOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stdoutOutputBuffer.split(b"\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stdoutData = b"\n".join(lines[:-1]) + b"\n"
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = lines[-1]
+                            # print("[+] newline in __stdoutOutputBuffer")
+                            # print(" | __stdoutData:",__stdoutData)
+                            # print(" | __stdoutOutputBuffer:",__stdoutOutputBuffer)
+
+                        if len(__stdoutData) != 0:
+                            # There is data to print
+                            try:
+                                sys.stdout.write(__stdoutData.decode(CODEC))
+                                sys.stdout.flush()
+                                __stdoutData = b""
+                            except UnicodeDecodeError:
+                                logging.error('Decoding error detected, consider running chcp.com at the target,\nmap the result with '
+                                              'https://docs.python.org/3/library/codecs.html#standard-encodings\nand then execute smbexec.py '
+                                              'again with -codec and the corresponding codec')
+                                print(__stdoutData.decode(CODEC, errors='replace'))
+                                __stdoutData = b""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = b""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except:
+                        pass
+        else:
+            __stdoutOutputBuffer, __stdoutData = "", ""
+
+            while True:
+                try:
+                    stdout_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
+                except:
+                    pass
+                else:
+                    try:
+                        if stdout_ans != LastDataSent:
+                            if len(stdout_ans) != 0:
+                                # Append new data to the buffer while there is data to read
+                                __stdoutOutputBuffer += stdout_ans
+
+                        promptRegex = r'([a-zA-Z]:[\\\/])((([a-zA-Z0-9 -\.]*)[\\\/]?)+(([a-zA-Z0-9 -\.]+))?)?>$'
+
+                        endsWithPrompt = bool(re.match(promptRegex, __stdoutOutputBuffer) is not None)
+                        if endsWithPrompt:
+                            # All data, we shouldn't have encoding errors
+                            # Adding a space after the prompt because it's beautiful
+                            __stdoutData = __stdoutOutputBuffer + " "
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = ""
+
+                        elif '\n' in __stdoutOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stdoutOutputBuffer.split("\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stdoutData = "\n".join(lines[:-1]) + "\n"
+                            # Remainder data for next iteration
+                            __stdoutOutputBuffer = lines[-1]
+
+                        if len(__stdoutData) != 0:
+                            # There is data to print
+                            sys.stdout.write(__stdoutData.decode(CODEC))
+                            sys.stdout.flush()
+                            __stdoutData = ""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = ""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except Exception as e:
+                        pass
+
 
 class RemoteStdErrPipe(Pipes):
     def __init__(self, transport, pipe, permisssions):
@@ -289,17 +385,87 @@ class RemoteStdErrPipe(Pipes):
 
     def run(self):
         self.connectPipe()
-        while True:
-            try:
-                ans = self.server.readFile(self.tid,self.fid, 0, 1024)
-            except:
-                pass
-            else:
+
+        if PY3:
+            __stderrOutputBuffer, __stderrData = b'', b''
+
+            while True:
                 try:
-                    sys.stderr.write(str(ans))
-                    sys.stderr.flush()
+                    stderr_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
                 except:
                     pass
+                else:
+                    try:
+                        if len(stderr_ans) != 0:
+                            # Append new data to the buffer while there is data to read
+                            __stderrOutputBuffer += stderr_ans
+
+                        if b'\n' in __stderrOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stderrOutputBuffer.split(b"\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stderrData = b"\n".join(lines[:-1]) + b"\n"
+                            # Remainder data for next iteration
+                            __stderrOutputBuffer = lines[-1]
+
+                        if len(__stderrData) != 0:
+                            # There is data to print
+                            try:
+                                sys.stdout.write(__stderrData.decode(CODEC))
+                                sys.stdout.flush()
+                                __stderrData = b""
+                            except UnicodeDecodeError:
+                                logging.error('Decoding error detected, consider running chcp.com at the target,\nmap the result with '
+                                              'https://docs.python.org/3/library/codecs.html#standard-encodings\nand then execute smbexec.py '
+                                              'again with -codec and the corresponding codec')
+                                print(__stderrData.decode(CODEC, errors='replace'))
+                                __stderrData = b""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = b""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except Exception as e:
+                        pass
+        else:
+            __stderrOutputBuffer, __stderrData = '', ''
+
+            while True:
+                try:
+                    stderr_ans = self.server.readFile(self.tid, self.fid, 0, 1024)
+                except:
+                    pass
+                else:
+                    try:
+                        if len(stderr_ans) != 0:
+                            # Append new data to the buffer while there is data to read
+                            __stderrOutputBuffer += stderr_ans
+
+                        if '\n' in __stderrOutputBuffer:
+                            # We have read a line, print buffer if it is not empty
+                            lines = __stderrOutputBuffer.split("\n")
+                            # All lines, we shouldn't have encoding errors
+                            __stderrData = "\n".join(lines[:-1]) + "\n"
+                            # Remainder data for next iteration
+                            __stderrOutputBuffer = lines[-1]
+
+                        if len(__stderrData) != 0:
+                            # There is data to print
+                            sys.stdout.write(__stderrData.decode(CODEC))
+                            sys.stdout.flush()
+                            __stderrData = ""
+                        else:
+                            # Don't echo the command that was sent, and clear it up
+                            LastDataSent = ""
+                        # Just in case this got out of sync, i'm cleaning it up if there are more than 10 chars,
+                        # it will give false positives tho.. we should find a better way to handle this.
+                        # if LastDataSent > 10:
+                        #     LastDataSent = ''
+                    except:
+                        pass
+
 
 class RemoteShell(cmd.Cmd):
     def __init__(self, server, port, credentials, tid, fid, share, transport):
@@ -330,8 +496,8 @@ class RemoteShell(cmd.Cmd):
         print("""
  lcd {path}                 - changes the current local directory to {path}
  exit                       - terminates the server process (and this session)
- put {src_file, dst_path}   - uploads a local file to the dst_path RELATIVE to the connected share (%s)
- get {file}                 - downloads pathname RELATIVE to the connected share (%s) to the current local dir
+ lput {src_file, dst_path}   - uploads a local file to the dst_path RELATIVE to the connected share (%s)
+ lget {file}                 - downloads pathname RELATIVE to the connected share (%s) to the current local dir
  ! {cmd}                    - executes a local shell cmd
 """ % (self.share, self.share))
         self.send_data('\r\n', False)
@@ -340,7 +506,7 @@ class RemoteShell(cmd.Cmd):
         os.system(s)
         self.send_data('\r\n')
 
-    def do_get(self, src_path):
+    def do_lget(self, src_path):
         try:
             if self.transferClient is None:
                 self.connect_transferClient()
@@ -357,7 +523,7 @@ class RemoteShell(cmd.Cmd):
 
         self.send_data('\r\n')
 
-    def do_put(self, s):
+    def do_lput(self, s):
         try:
             if self.transferClient is None:
                 self.connect_transferClient()
@@ -398,9 +564,9 @@ class RemoteShell(cmd.Cmd):
 
     def default(self, line):
         if PY3:
-            self.send_data(line.encode('cp437')+b'\r\n')
+            self.send_data(line.encode(CODEC)+b'\r\n')
         else:
-            self.send_data(line.decode(sys.stdin.encoding).encode('cp437')+'\r\n')
+            self.send_data(line.decode(sys.stdin.encoding).encode(CODEC)+'\r\n')
 
     def send_data(self, data, hideOutput = True):
         if hideOutput is True:
@@ -435,6 +601,11 @@ if __name__ == '__main__':
     parser.add_argument('-file', action='store', help="alternative RemCom binary (be sure it doesn't require CRT)")
     parser.add_argument('-ts', action='store_true', help='adds timestamp to every logging output')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+    parser.add_argument('-codec', action='store', help='Sets encoding used (codec) from the target\'s output (default '
+                                                       '"%s"). If errors are detected, run chcp.com at the target, '
+                                                       'map the result with '
+                          'https://docs.python.org/3/library/codecs.html#standard-encodings and then execute smbexec.py '
+                          'again with -codec and the corresponding codec ' % CODEC)
 
     group = parser.add_argument_group('authentication')
 
@@ -471,6 +642,12 @@ if __name__ == '__main__':
     # Init the example's logger theme
     logger.init(options.ts)
 
+    if options.codec is not None:
+        CODEC = options.codec
+    else:
+        if CODEC is None:
+            CODEC = 'utf-8'
+
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
         # Print the Library's installation path
@@ -478,15 +655,7 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
-
-    domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        options.target).groups('')
-
-    #In case the password contains '@'
-    if '@' in remoteName:
-        password = password + '@' + remoteName.rpartition('@')[0]
-        remoteName = remoteName.rpartition('@')[2]
+    domain, username, password, remoteName = parse_target(options.target)
 
     if domain is None:
         domain = ''
