@@ -1,26 +1,30 @@
-#!/usr/local/bin/python3.9
-#!/usr/local/bin/python3.9
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+#!/usr/local/bin/python3.12
+#!/usr/local/bin/python3.12
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
+# Description:
+#   [MS-RDPBCGR] and [MS-CREDSSP] partial implementation
+#   just to reach CredSSP auth. This example test whether
+#   an account is valid on the target host.
+#
 # Author:
 #  Alberto Solino (@agsolino)
 #
-# Description: [MS-RDPBCGR] and [MS-CREDSSP] partial implementation 
-#              just to reach CredSSP auth. This example test whether
-#              an account is valid on the target host.
-#
 # ToDo:
-#    [x] Manage to grab the server's SSL key so we can finalize the whole
+#   [x] Manage to grab the server's SSL key so we can finalize the whole
 #        authentication process (check [MS-CSSP] section 3.1.5)
 #
 
 from struct import pack, unpack
 
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket.structure import Structure
 from impacket.spnego import GSSAPI, ASN1_SEQUENCE, ASN1_OCTET_STRING, asn1decode, asn1encode
 
@@ -278,7 +282,7 @@ if __name__ == '__main__':
     import sys
     import logging
     from binascii import a2b_hex
-    from Crypto.Cipher import ARC4
+    from Cryptodome.Cipher import ARC4
     from impacket import ntlm, version
     try:
         from OpenSSL import SSL, crypto
@@ -407,7 +411,7 @@ if __name__ == '__main__':
 
        # Switching to TLS now
        ctx = SSL.Context(SSL.TLSv1_2_METHOD)
-       ctx.set_cipher_list('RC4,AES')
+       ctx.set_cipher_list(b'RC4,AES')
        tls = SSL.Connection(ctx,s)
        tls.set_connect_state()
        tls.do_handshake()
@@ -559,13 +563,7 @@ if __name__ == '__main__':
  
     options = parser.parse_args()
 
-    import re
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(options.target).groups('')
-
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    domain, username, password, address = parse_target(options.target)
 
     if domain is None:
         domain = ''
